/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.mail.internal.iterator.factory;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Provider;
import javax.inject.Singleton;
import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;

import org.xwiki.component.annotation.Component;
import org.xwiki.component.manager.ComponentManager;
import org.xwiki.mail.MimeMessageFactory;
import org.xwiki.mail.internal.iterator.UsersMimeMessageIterator;
import org.xwiki.model.reference.DocumentReference;

/**
 *
 * @version $Id: bc0b5ee4debfe39da117c3aec2c083f1afbf946b $
 * @since 6.4M3
 */
@Component
@Singleton
public class DefaultUsersMimeMessageIteratorFactory implements UsersMimeMessageIteratorFactory
{
    @Inject
    @Named("context")
    private Provider<ComponentManager> componentManagerProvider;

    @Override
    public Iterator<MimeMessage> create(List<DocumentReference> userReferences, MimeMessageFactory factory,
        Map<String, Object> parameters) throws MessagingException
    {
        ComponentManager cm = componentManagerProvider.get();
        UsersMimeMessageIterator iterator = new UsersMimeMessageIterator(userReferences, factory, parameters, cm);
        return iterator;
    }
}
